//+----------------------------------------------------------------------------------+
//|                                                                                  |
//|                                Levels x8.mq4                                     |
//|                                                                                  |
//+----------------------------------------------------------------------------------+

#property copyright "Doblece, Domas4 & Traderathome, Copyright @ 2009"
#property link      "email:  traderathome@msn.com"

/*------------------------------------------------------------------------------------
Levels x8:

     This indicator will place up to 8 lines on your charts, at prime levels of your 
choosing.  You can have the pattern repeated as many times as you like.  In other
words, the range of lines can then be repeated both above and below the initial range
as many times as you like.  Or, you can select zero additional ranges and the initial 
range will automatically fill below or above the next whole number as the price moves 
there.

     The lines can individually be of any available color and style, and of any 
thickness for solid lines.  You can turn on/off each line individually and in groups.  
When lines are displayed full screen wide, you can turn on/off the right margin price 
labels.  You can select to have the lines stop at the current candle.  Lines can be 
subordinated.  To enter levels, just enter the last two digits of the price.  You can
set linestyles to your favorite non-solid style and leave that setting.  When you 
want solid lines, simply enter a thickness greater than "1" and you will get a solid
line of that thickness.  Of course, if you want a single thickness solid line, you 
will have to change the linestyle to "0".

     The labels for the lines can be of any color, font size and font style.  You can
give the labels a name of your choice (example:  Quarter Level) and include the price.
Labels can be positioned from right of the current candle (0), to far left (100), and
they can be subordinated.  Zooming in and out on the chart will displace the labels, 
but with the next data tic, their selected position is restored.

     The indicator has an ON/OFF switch so you can turn the indicator off when the
display of the prime levels is not desired, yet preserve your settings for the chart.
You can also enter a time frame, above which the indicator will automatically not
display.  This enables switching to larger time frames for analysis, and having a 
chart clear of this indicator's display.   
---------------------------------------------------------------------------------------
ACKNOWLEDGEMENTS:

Doblece:
For the code enabling the indicator to work with platforms with unconventional
Point digits number (0.00001 and 0.001).      

Domas4:
For the code enabling the indicator to be set to automatically not display above
any time frame, as set by the user.
  
Shimodax:
For the idea and some core code to get started with (ref: SweetSpots.mq4).
----------------------------------------------------------------------------------------
Suggested Settings:                   White Chart                Black Chart  

Label Color                           MidnightBlue               DarkGray
Line1_Color                           MistyRose                  C'145,0,0'
Line2_Color                           C'211,248,211'             DarkGreen
Line3_Color                           Lavender                   C'73,73,73'
Line4_Color                           Lavender                   C'73,73,73'
Line5_Color                           Gainsboro                  C'92,75,44'
Line6_Color                           Gainsboro                  C'92,75,44'  
Line7_Color                           Gainsboro                  C'92,75,44'
Line8_Color                           Gainsboro                  C'92,75,44'  
                                                                                                                                                                                         
--------------------------------------------------------------------------------------*/  

#property indicator_chart_window

extern bool   Indicator_ON                    = true;
extern int    AddRangesAbove\BelowCurrent     = 6;
extern string TF_Choices                      = "M1, M5, M15, M30, H1, H4, D, W, M";
extern string Max_Display_TF                  = "H1";

extern string _                               = "";
extern string Part_1                          = "Line Settings:";
extern bool   EndLines_At_Current_Candle      = false;
extern bool   FScrLines_With_Margin_Prices    = false;
extern bool   Subordinate_Lines               = true;
extern bool   Show_Quarter_Lines              = true;
extern bool   Show_Eighths_Lines              = true;

extern string __                              = "";
extern string Part_2                          = "Label Settings:";
extern color  Color                           = MidnightBlue;
extern bool   ShowLabels                      = false;
extern string FontStyle                       = "Arial Bold";
extern int    FontSize                        = 12;          
extern bool   Line_Prices                     = true;
extern string Text_For_Label                  = "";
extern int    Shift_Labels_PerCent_Left       = 0;    
extern bool   Subordinate_Labels              = false;

extern string ___                             = "";
extern string Part_3                          = "Individual Line Configurations:";
extern color  Line1_Color                     = MistyRose; //Tomato
extern bool   Line1_On                        = true;
extern int    Line1_Last_Two_Digits           = 0;
extern int    Line1_Style_01234               = 2;
extern int    Line1_SolidThickness            = 3;

extern color  Line2_Color                     = C'211,248,211'; //Green; 
extern bool   Line2_On                        = true;
extern int    Line2_Last_Two_Digits           = 50;
extern int    Line2_Style_01234               = 2;
extern int    Line2_SolidThickness            = 3;

extern color  Line3_Color                     = Lavender; //DarkSlateBlue;
extern bool   Line3_On                        = true; 
extern int    Line3_Last_Two_Digits           = 25;
extern int    Line3_Style_01234               = 2;
extern int    Line3_SolidThickness            = 2;

extern color  Line4_Color                     = Lavender; //DarkSlateBlue;
extern bool   Line4_On                        = true;
extern int    Line4_Last_Two_Digits           = 75;
extern int    Line4_Style_01234               = 2;
extern int    Line4_SolidThickness            = 2;

extern color  Line5_Color                     = C'225,215,193'; //LightGray; 
extern bool   Line5_On                        = true;
extern int    Line5_Last_Two_Digits           = 12;
extern int    Line5_Style_01234               = 0;
extern int    Line5_SolidThickness            = 1;

extern color  Line6_Color                     = C'225,215,193'; //LightGray;
extern bool   Line6_On                        = true;
extern int    Line6_Last_Two_Digits           = 88;
extern int    Line6_Style_01234               = 0;
extern int    Line6_SolidThickness            = 1;

extern color  Line7_Color                     = C'225,215,193'; //LightGray;
extern bool   Line7_On                        = true;
extern int    Line7_Last_Two_Digits           = 38;
extern int    Line7_Style_01234               = 0;
extern int    Line7_SolidThickness            = 1;

extern color  Line8_Color                     = C'225,215,193'; //LightGray;
extern bool   Line8_On                        = true;
extern int    Line8_Last_Two_Digits           = 62;
extern int    Line8_Style_01234               = 0;
extern int    Line8_SolidThickness            = 1;

//Global Buffers, Variables, Etc.
double        Poin;
bool          FLAG_deinit;

//+-------------------------------------------------------------------------------------------+
//| Initialization                                                                            |
//+-------------------------------------------------------------------------------------------+
int init()
{
   FLAG_deinit = false; 
   //Checking for unconventional Point digits number
   if (Point == 0.00001) Poin = 0.0001; //5 digits
   else if (Point == 0.001) Poin = 0.01; //3 digits
   else Poin = Point; //Normal
   return(0);
}

//+-------------------------------------------------------------------------------------------+
//| De-initialization                                                                         |
//+-------------------------------------------------------------------------------------------+
int deinit()  
{
   int obj_total= ObjectsTotal();  
   for (int j= obj_total; j>=0; j--) 
      {
      string name= ObjectName(j);    
      if (StringSubstr(name,0,11)=="[Levels x8]"){ObjectDelete(name);}
      }      
   return(0);
}

//+-------------------------------------------------------------------------------------------+
//| Start                                                                                     |
//+-------------------------------------------------------------------------------------------+
int start()
{
   //If Indicator is "Off" deinitialize only once, not every tick------------------------------ 
   if(!Indicator_ON) 
      {
      if (!FLAG_deinit) {deinit(); FLAG_deinit = true;}
      return(0);
      }
   //If indicator is "ON", deinitialize and run------------------------------------------------   
   deinit(); FLAG_deinit = false; 
      
   //Exit if period is greater than  
   int E;if(Max_Display_TF== "M1") {E= 1;}
   else {if(Max_Display_TF== "M5") {E= 5;}
   else {if(Max_Display_TF== "M15"){E= 15;}
   else {if(Max_Display_TF== "M30"){E= 30;}
   else {if(Max_Display_TF== "H1") {E= 60;}
   else {if(Max_Display_TF== "H4") {E= 240;}
   else {if(Max_Display_TF== "D")  {E= 1440;}
   else {if(Max_Display_TF== "W")  {E= 10080;}
   else {if(Max_Display_TF== "M")  {E= 43200;}}}}}}}}}
   if(Period() > E) {deinit(); return(-1);}               

   //space out updates
   static datetime timelastupdate= 0; static datetime lasttimeframe= 0;     
   if (CurTime()-timelastupdate < 5 && Period()==lasttimeframe) return (0);
  
   //initialize needed variables
   int i, ssp, ssp1;  //choosing "int" instead of "double" drops decimal portion of value
   double linelevel, linestyle ,linewidth; color linecolor;
   ssp1= Bid;  //bid w/o decimal, 117 instead of 117.66
   ssp1= Bid / Poin;  //Point restores bid integers w/o the decimal, 11766 
   
   //initializing last two digits user entered  
   int u1 = Line1_Last_Two_Digits; 
   int u2 = Line2_Last_Two_Digits;
   int u3 = Line3_Last_Two_Digits;
   int u4 = Line4_Last_Two_Digits;
   int u5 = Line5_Last_Two_Digits;
   int u6 = Line6_Last_Two_Digits;
   int u7 = Line7_Last_Two_Digits;
   int u8 = Line8_Last_Two_Digits;         
   int c1=ssp1%100;  
   double LA, LB, LN; LN=100-c1; LB=c1; LA=LN; //if (c1<50)LB=c1;LA=LN;//if (c1>+50)LB=c1;LA=LN;Comment(LB);

   //calculate line levels for user specified # of ranges
   ssp=ssp1; //start ssp at reference bid
   for (i= -((100*(AddRangesAbove\BelowCurrent))+LB); i<=((100*(AddRangesAbove\BelowCurrent))+LA); i++)   
      {
         ssp=ssp1+i;  //Comment(i+"  "+ssp);
         c1=ssp%100;  //Comment(c1);
         if (c1==u1 && Line1_On == true)
            {           
            linestyle = Line1_Style_01234;
            linecolor = Line1_Color;
            linewidth = Line1_SolidThickness;
            linelevel = ssp*Poin;
            }
         if (c1==u2 && Line2_On == true)
            {           
            linestyle = Line2_Style_01234;
            linecolor = Line2_Color;
            linewidth = Line2_SolidThickness;
            linelevel = ssp*Poin;
            }
         if (c1==u3 && Line3_On && Show_Quarter_Lines)
            {           
            linestyle = Line3_Style_01234;
            linecolor = Line3_Color;
            linewidth = Line3_SolidThickness;
            linelevel = ssp*Poin;
            }
         if (c1==u4 && Line4_On && Show_Quarter_Lines)
            {           
            linestyle = Line4_Style_01234;
            linecolor = Line4_Color;
            linewidth = Line4_SolidThickness;
            linelevel = ssp*Poin;
            }
         if (c1==u5 && Line5_On && Show_Eighths_Lines)
            {           
            linestyle = Line5_Style_01234;
            linecolor = Line5_Color;
            linewidth = Line5_SolidThickness;
            linelevel = ssp*Poin;
            }
         if (c1==u6 && Line6_On && Show_Eighths_Lines)
            {           
            linestyle = Line6_Style_01234;
            linecolor = Line6_Color;
            linewidth = Line6_SolidThickness;
            linelevel = ssp*Poin;
            }
         if (c1==u7 && Line7_On && Show_Eighths_Lines)
            {           
            linestyle = Line7_Style_01234;
            linecolor = Line7_Color;
            linewidth = Line7_SolidThickness;
            linelevel = ssp*Poin;
            }
         if (c1==u8 && Line8_On && Show_Eighths_Lines)
            {           
            linestyle = Line8_Style_01234;
            linecolor = Line8_Color;
            linewidth = Line8_SolidThickness;
            linelevel = ssp*Poin;
            }
            
         //branch to subroutine to draw lines
         SetLevel(DoubleToStr(linelevel,Digits), linelevel,  linecolor, linestyle, linewidth, Time[10]);
         
         //branch to subroutine to draw labels
         if(ShowLabels == true){SetLabel(DoubleToStr(linelevel,Digits), linelevel, Color);}
      }
     
   return(0);
   }
   
//+-------------------------------------------------------------------------------------------+
//| Sub-routine to name and draw lines                                                        |                                                                                  |
//+---------------------------------------------------------------------------------------- --+
void SetLevel(string text, double level, color linecolor, int linestyle, int linewidth, datetime startofday)
{
   if (linewidth > 1) linestyle = 0;
   int digits=Digits;  
   int startline = WindowFirstVisibleBar(); 
   int endline = Time[0];
   int G = false;
   int Z= OBJ_TREND;
   string linename= "[Levels x8] " + StringTrimLeft(text) + " Line"; 
     
   // create or move the horizontal line 
   if( FScrLines_With_Margin_Prices == true){Z = OBJ_HLINE;}
   if (EndLines_At_Current_Candle) {startline= Time[0]; endline= WindowFirstVisibleBar(); Z= OBJ_TREND;} 
   if (Subordinate_Lines) {G = true;} 
   if (ObjectFind(linename) != 0)
      {
      ObjectCreate(linename,  Z, 0, startline, level, endline, level); 
      ObjectSet(linename, OBJPROP_STYLE, linestyle);     
      ObjectSet(linename, OBJPROP_COLOR, linecolor);
      ObjectSet(linename, OBJPROP_WIDTH, linewidth);
      ObjectSet(linename, OBJPROP_BACK, G); 
      }
   else 
      {
      ObjectMove(linename, 0, startline, level);
      ObjectMove(linename, 1, endline, level);
      }
}

//+-------------------------------------------------------------------------------------------+
//| Sub-routine to name and draw labels                                                       |                                                                                  |
//+---------------------------------------------------------------------------------------- --+
void SetLabel(string text, double linelevel, color Color_Labels)
{
   int ml = ((WindowFirstVisibleBar()-0) * Shift_Labels_PerCent_Left)/ 100; 
   int G = false;
   string Font;
   string name = "[Levels x8] " + StringTrimLeft(text) + " Label";
   string TEXT = Text_For_Label;
   if (Line_Prices == true){TEXT = TEXT + "   "+DoubleToStr(linelevel, Digits);}  
   if (Subordinate_Labels) {G = true;} 
   
   for (int i = StringLen(TEXT)*2+2; i >= 0; i--) string tab = tab + " ";
   
   if(ObjectFind(name) != 0)
      {
      ObjectCreate(name, OBJ_TEXT, 0, Time[ml], linelevel);       
      ObjectSet(name, OBJPROP_BACK, G);
      ObjectSetText(name, tab+TEXT, FontSize, FontStyle, Color_Labels);
      }
      else
      {
      ObjectMove(name, 0, Time[ml], linelevel);
      }  
}

//+-------------------------------------------------------------------------------------------+
//| End                                                                                       |                                                        
//+-------------------------------------------------------------------------------------------+
    