//+------------------------------------------------------------------------------------------+
//|                                                                                          |
//|                                Market Panel Clock.mq4                                    |
//|                                                                                          |
//+------------------------------------------------------------------------------------------+
#property copyright "pips4life" //P4L Clock.mq4, a rewrite of Clock.mq4, Clock_v1_3.mq4
#property link      "pips4life, a user at forexfactory.com"

/*--------------------------------------------------------------------------------------------
Market Panel Clock Overview:

This trimmed down version of the P4L Clock indicator is designed to be used with another 
indicator that places a data panel in the upper left corner of the chart.  It creates an 
extension to that panel containing the P4L Clock.  It contains only the P4L clock code for
time/location displays, and is intended to display only 7 out of a list of 21 choices.  If
more than 7 are set to display, the extension panel will expand accordingly, but will no
longer align with the primary data panel as intended. 
                                                                             - Traderathome
--------------------------------------------------------------------------------------------*/                                                                
#property indicator_chart_window     

#include <stdlib.mqh>
//See function documentation at http://msdn.microsoft.com/en-us/library/ms725473(VS.85).aspx
#import "kernel32.dll"
void GetLocalTime(int& LocalTimeArray[]);
void GetSystemTime(int& systemTimeArray[]);
int  GetTimeZoneInformation(int& LocalTZInfoArray[]);
bool SystemTimeToTzSpecificLocalTime(int& targetTZinfoArray[], 
     int& systemTimeArray[], int& targetTimeArray[]);
#import
/*--------------------------------------------------------------------------------------------
Suggested Settings:               White Chart           Black Chart   

Location_MktClosed                Black                 C'36,151,36'
Clock_MktClosed                   Black                 C'0,122,244'
Location_MktOpen                  Crimson               Red 
Clock_MktOpen                     MediumBlue            DarkOrange 
Background                        C'226,226,250         C'23,23,23'
---------------------------------------------------------------------------------------------*/

//---- input parameters -- FYI, THE USER MAY CUSTOMIZE THESE EXTERN VARIABLE SETTINGS AS DESIRED:
extern bool     Indicator_ON                  = true;          
extern color    Location_MktClosed            = Black;        
extern color    Clock_MktClosed               = Black;  
extern color    Location_MktOpen              = Crimson;     
extern color    Clock_MktOpen                 = MediumBlue;  
extern color    Background                    = C'226,226,250';
extern bool     Display_Times_With_AMPM       = false;   
extern string   Note_                         = "Choose Only 7 From 21 Following:";
extern bool     ShowBroker                    = true;
extern bool     ShowAuckland                  = false;
extern bool     ShowSydney                    = false;
extern bool     ShowTokyo                     = true;
extern bool     ShowChina                     = true;
extern bool     ShowJakarta                   = false;
extern bool     ShowIndia                     = false;
extern bool     ShowDubai                     = false;
extern bool     ShowMoscow                    = true;
extern bool     ShowIsrael                    = false;
extern bool     ShowHelsinki                  = false;
extern bool     ShowBerlin                    = true;
extern bool 	 ShowLondon                    = true;
extern bool 	 ShowUTC_GMT                   = false;//No GMT change with DST. Summer London=GMT, winter London=GMT+1.
extern bool     ShowBrazil                    = false;
extern bool 	 ShowNewYork                   = true;
extern bool     ShowCentral                   = false;
extern bool     ShowMexico                    = false;
extern bool     ShowMountain                  = false;
extern bool 	 ShowPacific                   = false;
extern bool     ShowLocal                     = false;//FYI, this is your Local computer clock.
      
//---- buffers
bool   FLAG_deinit;
int    obj_total, k;
double bfactor;
string backName = "[MPClock] _Background"; //"_" critical to placing bacground on chart first, beneath clock text.
string name;
int    AucklandTZInfoArray[43];
int    SydneyTZInfoArray[43];
int    TokyoTZInfoArray[43];
int    ChinaTZInfoArray[43];
int    JakartaTZInfoArray[43];
int    IndiaTZInfoArray[43];
int    DubaiTZInfoArray[43];
int    MoscowTZInfoArray[43];
int    IsraelTZInfoArray[43];
int    HelsinkiTZInfoArray[43];
int    BerlinTZInfoArray[43];
int    LondonTZInfoArray[43];
int    BrazilTZInfoArray[43];
int    NewYorkTZInfoArray[43];
int    CentralTZInfoArray[43];
int    MexicoTZInfoArray[43];
int    MountainTZInfoArray[43];
int    PacificTZInfoArray[43];
int    LocalTZInfoArray[43];

int      VerticalOffsetPixels          = 3; //for text, not background     
int      HorizClockOffsetPixels        = 156; //150, 156 
int      HorizTimezoneOffsetPixels     = 176; //170, 176, 190  

string   FontName                      = "Arial";    
int      TimezoneFontSize              = 8;       
int      ClockFontSize                 = 8;         
int      LineSpacing                   = 12;        
  
bool 	   Display_Time_With_Seconds     = false;   

int      LocalOpenHour                 = 8;       
int      LocalCloseHour                = 17;
int      SydneyLocalOpenHour           = 7;          
int      SydneyLocalCloseHour          = 16;  

bool     Background_Under_Labels       = true; 
int      Background_AddWidth_Pixels; //= 40; //43;  


//+------------------------------------------------------------------------------------------+
//| Custom Indicator Initialization                                                          |
//+------------------------------------------------------------------------------------------+
int init()
  {   
   //With the first DLL call below, the program will exit (and stop) automatically after one alert. 
   if ( !IsDllsAllowed() ) 
      {
      Alert( WindowExpertName(),": Error! Check 'Allow DLL Imports' in indicator Common Tab.");   
      }
   
   FLAG_deinit = false;
     
   GetAllTimeZoneInfo(AucklandTZInfoArray, SydneyTZInfoArray, TokyoTZInfoArray, ChinaTZInfoArray, 
      JakartaTZInfoArray, IndiaTZInfoArray, DubaiTZInfoArray, MoscowTZInfoArray, IsraelTZInfoArray, 
      HelsinkiTZInfoArray, BerlinTZInfoArray, LondonTZInfoArray, BrazilTZInfoArray, NewYorkTZInfoArray, 
      CentralTZInfoArray, MexicoTZInfoArray, MountainTZInfoArray, PacificTZInfoArray, LocalTZInfoArray);
      
   //HorizTimezoneOffsetPixels  = 170+6;
   Background_AddWidth_Pixels = 44;
 	bfactor =  1.12; //decreasing extends bottom downward   
 		   
   int top=VerticalOffsetPixels;
   int left = HorizTimezoneOffsetPixels;
   int right = HorizClockOffsetPixels;
   int adjustLeft = 20;

   if (Display_Times_With_AMPM)
      {
      //if (!Display_Time_With_Seconds) adjustLeft = MathMax(adjustLeft,26); // 5+3=8 characters HH:MM AM
      //else adjustLeft = MathMax(adjustLeft,40); // 5+6=11 characters HH:MM:SS AM
      adjustLeft = MathMax(adjustLeft,40); // 5+6=11 characters HH:MM:SS AM
      HorizTimezoneOffsetPixels = 190;
      Background_AddWidth_Pixels = 38;
      bfactor = 1.16;  
      }

   left = left + adjustLeft;   
   int offset=0;
   int revoffset=0;

   if(ShowBroker)
      {
      ObjectMakeLabel( "[MPClock] Brokerl", left, top+offset );
      ObjectMakeLabel( "[MPClock] Brokert", right, top+offset );
     	offset+=LineSpacing;
      }
  	// Begin world timezones, listed IN ORDER!
   if(ShowAuckland)
      {
   	ObjectMakeLabel( "[MPClock] Aucklandl", left, top+offset );
   	ObjectMakeLabel( "[MPClock] Aucklandt", right, top+offset );
   	offset+=LineSpacing;
      }
   if(ShowSydney)
      {
   	ObjectMakeLabel( "[MPClock] Sydneyl", left, top+offset );
   	ObjectMakeLabel( "[MPClock] Sydneyt", right, top+offset );
   	offset+=LineSpacing;
      }
   if(ShowTokyo)
      {
   	ObjectMakeLabel( "[MPClock] Tokyol", left, top+offset );
   	ObjectMakeLabel( "[MPClock] Tokyot", right, top+offset );
   	offset+=LineSpacing;
      }
   if(ShowChina)
      {
   	ObjectMakeLabel( "[MPClock] Chinal", left, top+offset );
   	ObjectMakeLabel( "[MPClock] Chinat", right, top+offset );
   	offset+=LineSpacing;
      }
   if(ShowJakarta)
      {
   	ObjectMakeLabel( "[MPClock] Jakartal", left, top+offset );
   	ObjectMakeLabel( "[MPClock] Jakartat", right, top+offset );
   	offset+=LineSpacing;
      }
   if(ShowIndia)
      {
   	ObjectMakeLabel( "[MPClock] Indial", left, top+offset );
   	ObjectMakeLabel( "[MPClock] Indiat", right, top+offset );
   	offset+=LineSpacing;
      }
   if(ShowDubai)
      {
   	ObjectMakeLabel( "[MPClock] Dubail", left, top+offset );
   	ObjectMakeLabel( "[MPClock] Dubait", right, top+offset );
   	offset+=LineSpacing;
      }
   if(ShowMoscow)
      {
   	ObjectMakeLabel( "[MPClock] Moscowl", left, top+offset );
   	ObjectMakeLabel( "[MPClock] Moscowt", right, top+offset );
   	offset+=LineSpacing;
      }
   if(ShowIsrael)
      {
   	ObjectMakeLabel( "[MPClock] Israell", left, top+offset );
   	ObjectMakeLabel( "[MPClock] Israelt", right, top+offset );
   	offset+=LineSpacing;
      }
   if(ShowHelsinki)
      {
   	ObjectMakeLabel( "[MPClock] Helsinkil", left, top+offset );
   	ObjectMakeLabel( "[MPClock] Helsinkit", right, top+offset );
   	offset+=LineSpacing;
      }
   if(ShowBerlin)
      {
   	ObjectMakeLabel( "[MPClock] Berlinl", left, top+offset );
   	ObjectMakeLabel( "[MPClock] Berlint", right, top+offset );
   	offset+=LineSpacing;
      }
   if(ShowLondon)
      {
	   ObjectMakeLabel( "[MPClock] Londonl", left, top+offset );
   	ObjectMakeLabel( "[MPClock] Londont", right, top+offset );
   	offset+=LineSpacing;
      }
   if(ShowUTC_GMT)
      {
   	ObjectMakeLabel( "[MPClock] utcl", left, top+offset );
   	ObjectMakeLabel( "[MPClock] utct", right, top+offset );
   	offset+=LineSpacing;
      }
   if(ShowBrazil)
      {
   	ObjectMakeLabel( "[MPClock] Brazill", left, top+offset );
   	ObjectMakeLabel( "[MPClock] Brazilt", right, top+offset );
   	offset+=LineSpacing;
      }
   if(ShowNewYork)
      {
	   ObjectMakeLabel( "[MPClock] NewYorkl", left, top+offset );
   	ObjectMakeLabel( "[MPClock] NewYorkt", right, top+offset );
   	offset+=LineSpacing;
      }
   if(ShowCentral)
      {
   	ObjectMakeLabel( "[MPClock] Centrall", left, top+offset );
   	ObjectMakeLabel( "[MPClock] Centralt", right, top+offset );
   	offset+=LineSpacing;
      }
   if(ShowMexico)
      {
   	ObjectMakeLabel( "[MPClock] Mexicol", left, top+offset );
   	ObjectMakeLabel( "[MPClock] Mexicot", right, top+offset );
   	offset+=LineSpacing;
      }
   if(ShowMountain)
      {
   	ObjectMakeLabel( "[MPClock] Mountainl", left, top+offset );
   	ObjectMakeLabel( "[MPClock] Mountaint", right, top+offset );
   	offset+=LineSpacing;
      }
   if(ShowPacific)
      {
	   ObjectMakeLabel( "[MPClock] Pacificl", left, top+offset );
   	ObjectMakeLabel( "[MPClock] Pacifict", right, top+offset );
   	offset+=LineSpacing;
      }
   if(ShowLocal)
      {
      ObjectMakeLabel( "[MPClock] Locall", left, top+offset );
      ObjectMakeLabel( "[MPClock] Localt", right, top+offset );
      offset+=LineSpacing;
      }

   // BACKGROUND must be LAST label added (because of offset value).
   if (Background_Under_Labels)
      {  
      // DONE with offset as used above.  Now reset it for background use.     
      offset = MathMax(offset,revoffset) + MathMax(0,VerticalOffsetPixels-5); //-5 Related to total height of background.
      int yB = MathMax(0,VerticalOffsetPixels -4);//-2, increase to lower top of background
      int xB = MathMax(1,HorizClockOffsetPixels -6);//-6, larger neg. # moves background to left
 	   //double bfactor = 1.16; // 1.1, increasing extends bottom downward
      string btext = "gg"; //"g" is box. Double "gg" is the width of approx. the bfactor * 2 *  Bfontsize
 	   int Bfontsize = (MathMax(left,right) - HorizClockOffsetPixels + Background_AddWidth_Pixels)/StringLen(btext);  
 	  	         
      // Create the first background label
      createBackground (backName,btext,Bfontsize,xB,yB);
      // Create overlapping rectangles (backName1-8) to create the background.
      // FYI, backName & backName1-5 are usually enough.  6-8 are just to make sure.                 
      for (int k=1; k<=1 ; k++ )
         { 
         if (offset <= MathCeil(yB+Bfontsize*bfactor)) break;
         yB = MathMin(yB+Bfontsize*bfactor, offset - Bfontsize*bfactor);
         createBackground (backName+k,btext,Bfontsize,xB,yB);
         }
      }

   return(0);
   } 

//+------------------------------------------------------------------------------------------+
//| Custom Indicator Deinitialization                                                        |
//+------------------------------------------------------------------------------------------+
int deinit()
   {
   obj_total= ObjectsTotal();  
   for (k= obj_total; k>=0; k--){
      name= ObjectName(k); 
      if (StringSubstr(name,0,9)=="[MPClock]"){ObjectDelete(name);}}
   //Comment("");     
   return(0);
   }
   
//+------------------------------------------------------------------------------------------+
//| Custom Indicator Start -- Runs with each new tick                                        |
//+------------------------------------------------------------------------------------------+
int start()
  {  
   //If Indicator is "Off" deinitialize only once, not every tick-----------------------------  
   if(!Indicator_ON) 
      {
      if (!FLAG_deinit) {deinit(); FLAG_deinit = true;}
      return(0);
      }
      
   //Otherwise, just proceed------------------------------------------------------------------   
   FLAG_deinit = false; 
 
   int    systemTimeArray[4];  
   int    AucklandTimeArray[4];
   int    SydneyTimeArray[4];
   int    TokyoTimeArray[4];
   int    ChinaTimeArray[4];
   int    JakartaTimeArray[4];
   int    IndiaTimeArray[4];
   int    DubaiTimeArray[4];
   int    MoscowTimeArray[4];
   int    IsraelTimeArray[4];
   int    HelsinkiTimeArray[4];
   int    BerlinTimeArray[4];
   int    LondonTimeArray[4];
   int    BrazilTimeArray[4];
   int    NewYorkTimeArray[4];
   int    CentralTimeArray[4];
   int    MexicoTimeArray[4];
   int    MountainTimeArray[4];
   int    PacificTimeArray[4];
   int    LocalTimeArray[4];   
   
   GetLocalTime(LocalTimeArray);
   datetime Local_Time = TimeArrayToTime(LocalTimeArray);

   datetime BrokerTime = TimeCurrent();
   datetime BrokerCorrection = 0;

   GetSystemTime(systemTimeArray);
   datetime UTC = TimeArrayToTime(systemTimeArray)+BrokerCorrection;

   SystemTimeToTzSpecificLocalTime(AucklandTZInfoArray, systemTimeArray, AucklandTimeArray);
   datetime Auckland   = TimeArrayToTime(AucklandTimeArray)+BrokerCorrection;
      
   SystemTimeToTzSpecificLocalTime(SydneyTZInfoArray, systemTimeArray, SydneyTimeArray);
   datetime Sydney  = TimeArrayToTime(SydneyTimeArray)+BrokerCorrection;
   
   SystemTimeToTzSpecificLocalTime(TokyoTZInfoArray, systemTimeArray, TokyoTimeArray);
   datetime Tokyo   = TimeArrayToTime(TokyoTimeArray)+BrokerCorrection;
      
   SystemTimeToTzSpecificLocalTime(ChinaTZInfoArray, systemTimeArray, ChinaTimeArray);
   datetime China   = TimeArrayToTime(ChinaTimeArray)+BrokerCorrection;
      
   SystemTimeToTzSpecificLocalTime(JakartaTZInfoArray, systemTimeArray, JakartaTimeArray);
   datetime Jakarta   = TimeArrayToTime(JakartaTimeArray)+BrokerCorrection;
      
   SystemTimeToTzSpecificLocalTime(IndiaTZInfoArray, systemTimeArray, IndiaTimeArray);
   datetime India   = TimeArrayToTime(IndiaTimeArray)+BrokerCorrection;
      
   SystemTimeToTzSpecificLocalTime(DubaiTZInfoArray, systemTimeArray, DubaiTimeArray);
   datetime Dubai   = TimeArrayToTime(DubaiTimeArray)+BrokerCorrection;
      
   SystemTimeToTzSpecificLocalTime(MoscowTZInfoArray, systemTimeArray, MoscowTimeArray);
   datetime Moscow   = TimeArrayToTime(MoscowTimeArray)+BrokerCorrection;
      
   SystemTimeToTzSpecificLocalTime(IsraelTZInfoArray, systemTimeArray, IsraelTimeArray);
   datetime Israel   = TimeArrayToTime(IsraelTimeArray)+BrokerCorrection;
      
   SystemTimeToTzSpecificLocalTime(HelsinkiTZInfoArray, systemTimeArray, HelsinkiTimeArray);
   datetime Helsinki   = TimeArrayToTime(HelsinkiTimeArray)+BrokerCorrection;
      
   SystemTimeToTzSpecificLocalTime(BerlinTZInfoArray, systemTimeArray, BerlinTimeArray);
   datetime Berlin   = TimeArrayToTime(BerlinTimeArray)+BrokerCorrection;
      
   SystemTimeToTzSpecificLocalTime(LondonTZInfoArray, systemTimeArray, LondonTimeArray);
   datetime London  = TimeArrayToTime(LondonTimeArray)+BrokerCorrection;
   
   SystemTimeToTzSpecificLocalTime(BrazilTZInfoArray, systemTimeArray, BrazilTimeArray);
   datetime Brazil  = TimeArrayToTime(BrazilTimeArray)+BrokerCorrection;
   
   SystemTimeToTzSpecificLocalTime(NewYorkTZInfoArray, systemTimeArray, NewYorkTimeArray);
   datetime NewYork = TimeArrayToTime(NewYorkTimeArray)+BrokerCorrection;
   
   SystemTimeToTzSpecificLocalTime(CentralTZInfoArray, systemTimeArray, CentralTimeArray);
   datetime Central   = TimeArrayToTime(CentralTimeArray)+BrokerCorrection;
      
   SystemTimeToTzSpecificLocalTime(MexicoTZInfoArray, systemTimeArray, MexicoTimeArray);
   datetime Mexico   = TimeArrayToTime(MexicoTimeArray)+BrokerCorrection;
      
   SystemTimeToTzSpecificLocalTime(MountainTZInfoArray, systemTimeArray, MountainTimeArray);
   datetime Mountain   = TimeArrayToTime(MountainTimeArray)+BrokerCorrection;
      
   SystemTimeToTzSpecificLocalTime(PacificTZInfoArray, systemTimeArray, PacificTimeArray);
   datetime Pacific   = TimeArrayToTime(PacificTimeArray)+BrokerCorrection;
   
   string Brokers = TimeToString( TimeCurrent() );
   string Aucklands = TimeToString( Auckland  );
   string Sydneys = TimeToString(Sydney);
   string Tokyos = TimeToString( Tokyo  );
   string Chinas = TimeToString( China  );
   string Jakartas = TimeToString( Jakarta  );
   string Indias = TimeToString( India  );
   string Dubais = TimeToString( Dubai  );
   string Moscows = TimeToString( Moscow  );
   string Israels = TimeToString( Israel  );
   string Helsinkis = TimeToString( Helsinki  );
   string Berlins = TimeToString( Berlin  );
   string Londons = TimeToString( London  );
   string UTCs = TimeToString( UTC );
   string Brazils = TimeToString( Brazil  );
   string NewYorks = TimeToString( NewYork  );
   string Centrals = TimeToString( Central  );
   string Mexicos = TimeToString( Mexico  );
   string Mountains = TimeToString( Mountain  );
   string Pacifics = TimeToString( Pacific  );
   string Locals = TimeToString( Local_Time  );

   if(ShowBroker)
      {
	   ObjectSetText( "[MPClock] Brokerl", "Broker", TimezoneFontSize, FontName, Location_MktClosed );
      ObjectSetText( "[MPClock] Brokert", Brokers, ClockFontSize, FontName, Clock_MktClosed );
      }
   if(ShowAuckland)
      {
   	if (TimeDayOfWeek(Auckland) != 0 && TimeDayOfWeek(Auckland) != 6 
   	&& TimeHour(Auckland) >= LocalOpenHour && TimeHour(Auckland) < LocalCloseHour)
      	{
   	   ObjectSetText( "[MPClock] Aucklandl", "Auckland", TimezoneFontSize, FontName, Location_MktOpen );
   	   ObjectSetText( "[MPClock] Aucklandt", Aucklands, ClockFontSize, FontName, Clock_MktOpen );
   	   }
   	else
   	   {
   	   ObjectSetText( "[MPClock] Aucklandl", "Auckland", TimezoneFontSize, FontName, Location_MktClosed );
   	   ObjectSetText( "[MPClock] Aucklandt", Aucklands, ClockFontSize, FontName, Clock_MktClosed );
      	}
      }
   if(ShowSydney)
      {
   	if (TimeDayOfWeek(Sydney) != 0 && TimeDayOfWeek(Sydney) != 6 
   	&& TimeHour(Sydney) >= SydneyLocalOpenHour && TimeHour(Sydney) < SydneyLocalCloseHour)
   	   {
   	   ObjectSetText( "[MPClock] Sydneyl", "Sydney", TimezoneFontSize, FontName, Location_MktOpen );
   	   ObjectSetText( "[MPClock] Sydneyt", Sydneys, ClockFontSize, FontName, Clock_MktOpen );
   	   }
   	else
   	   {
   	   ObjectSetText( "[MPClock] Sydneyl", "Sydney", TimezoneFontSize, FontName, Location_MktClosed );
   	   ObjectSetText( "[MPClock] Sydneyt", Sydneys, ClockFontSize, FontName, Clock_MktClosed );
      	}
      }
   if(ShowTokyo)
      {
   	if (TimeDayOfWeek(Tokyo) != 0 && TimeDayOfWeek(Tokyo) != 6 
   	&& TimeHour(Tokyo) >= LocalOpenHour && TimeHour(Tokyo) < LocalCloseHour)
   	   {
   	   ObjectSetText( "[MPClock] Tokyol", "Tokyo", TimezoneFontSize, FontName, Location_MktOpen );
   	   ObjectSetText( "[MPClock] Tokyot", Tokyos, ClockFontSize, FontName, Clock_MktOpen );
   	   }
   	else
   	   {
   	   ObjectSetText( "[MPClock] Tokyol", "Tokyo", TimezoneFontSize, FontName, Location_MktClosed );
   	   ObjectSetText( "[MPClock] Tokyot", Tokyos, ClockFontSize, FontName, Clock_MktClosed );
   	   }
      }
   if(ShowChina)
      {
   	if (TimeDayOfWeek(China) != 0 && TimeDayOfWeek(China) != 6 
   	&& TimeHour(China) >= LocalOpenHour && TimeHour(China) < LocalCloseHour)
   	   {
   	   ObjectSetText( "[MPClock] Chinal", "Beijing", TimezoneFontSize, FontName, Location_MktOpen );
   	   ObjectSetText( "[MPClock] Chinat", Chinas, ClockFontSize, FontName, Clock_MktOpen );
   	   }
   	else
   	   {
   	   ObjectSetText( "[MPClock] Chinal", "Beijing", TimezoneFontSize, FontName, Location_MktClosed );
   	   ObjectSetText( "[MPClock] Chinat", Chinas, ClockFontSize, FontName, Clock_MktClosed );
   	   }
      }
   if(ShowJakarta)
      {
   	if (TimeDayOfWeek(Jakarta) != 0 && TimeDayOfWeek(Jakarta) != 6
   	&& TimeHour(Jakarta) >= LocalOpenHour && TimeHour(Jakarta) < LocalCloseHour)
   	   {
   	   ObjectSetText( "[MPClock] Jakartal", "Jakarta", TimezoneFontSize, FontName, Location_MktOpen );
   	   ObjectSetText( "[MPClock] Jakartat", Jakartas, ClockFontSize, FontName, Clock_MktOpen );
   	   }
   	else
   	   {
   	   ObjectSetText( "[MPClock] Jakartal", "Jakarta", TimezoneFontSize, FontName, Location_MktClosed );
   	   ObjectSetText( "[MPClock] Jakartat", Jakartas, ClockFontSize, FontName, Clock_MktClosed );
   	   }
      }
   if(ShowIndia)
      {
   	if (TimeDayOfWeek(India) != 0 && TimeDayOfWeek(India) != 6 
   	&& TimeHour(India) >= LocalOpenHour && TimeHour(India) < LocalCloseHour)
   	   {
   	   ObjectSetText( "[MPClock] Indial", "India", TimezoneFontSize, FontName, Location_MktOpen );
   	   ObjectSetText( "[MPClock] Indiat", Indias, ClockFontSize, FontName, Clock_MktOpen );
   	   }
   	else
      	{
   	   ObjectSetText( "[MPClock] Indial", "India", TimezoneFontSize, FontName, Location_MktClosed );
   	   ObjectSetText( "[MPClock] Indiat", Indias, ClockFontSize, FontName, Clock_MktClosed );
         }
      }
   if(ShowDubai)
      {
   	if (TimeDayOfWeek(Dubai) != 0 && TimeDayOfWeek(Dubai) != 6 
   	&& TimeHour(Dubai) >= LocalOpenHour && TimeHour(Dubai) < LocalCloseHour)
      	{
   	   ObjectSetText( "[MPClock] Dubail", "Dubai", TimezoneFontSize, FontName, Location_MktOpen );
   	   ObjectSetText( "[MPClock] Dubait", Dubais, ClockFontSize, FontName, Clock_MktOpen );
   	   }
   	else
   	   {
   	   ObjectSetText( "[MPClock] Dubail", "Dubai", TimezoneFontSize, FontName, Location_MktClosed );
   	   ObjectSetText( "[MPClock] Dubait", Dubais, ClockFontSize, FontName, Clock_MktClosed );
      	}
      }
   if(ShowMoscow)
      {
   	if (TimeDayOfWeek(Moscow) != 0 && TimeDayOfWeek(Moscow) != 6 
   	&& TimeHour(Moscow) >= LocalOpenHour && TimeHour(Moscow) < LocalCloseHour)
   	   {
   	   ObjectSetText( "[MPClock] Moscowl", "Moscow", TimezoneFontSize, FontName, Location_MktOpen );
   	   ObjectSetText( "[MPClock] Moscowt", Moscows, ClockFontSize, FontName, Clock_MktOpen );
      	}
   	else
   	   {
   	   ObjectSetText( "[MPClock] Moscowl", "Moscow", TimezoneFontSize, FontName, Location_MktClosed );
   	   ObjectSetText( "[MPClock] Moscowt", Moscows, ClockFontSize, FontName, Clock_MktClosed );
   	   }
      }
   if(ShowIsrael)
      {
   	if (TimeDayOfWeek(Israel) != 0 && TimeDayOfWeek(Israel) != 6
   	&& TimeHour(Israel) >= LocalOpenHour && TimeHour(Israel) < LocalCloseHour)
      	{
   	   ObjectSetText( "[MPClock] Israell", "Israel", TimezoneFontSize, FontName, Location_MktOpen );
   	   ObjectSetText( "[MPClock] Israelt", Israels, ClockFontSize, FontName, Clock_MktOpen );
      	}
   	else
      	{
   	   ObjectSetText( "[MPClock] Israell", "Israel", TimezoneFontSize, FontName, Location_MktClosed );
   	   ObjectSetText( "[MPClock] Israelt", Israels, ClockFontSize, FontName, Clock_MktClosed );
   	   }
      }
   if(ShowHelsinki)
      {
   	if (TimeDayOfWeek(Helsinki) != 0 && TimeDayOfWeek(Helsinki) != 6 
   	&& TimeHour(Helsinki) >= LocalOpenHour && TimeHour(Helsinki) < LocalCloseHour)
      	{
   	   ObjectSetText( "[MPClock] Helsinkil", "Helsinki", TimezoneFontSize, FontName, Location_MktOpen );
   	   ObjectSetText( "[MPClock] Helsinkit", Helsinkis, ClockFontSize, FontName, Clock_MktOpen );
   	   }
   	else
   	   {
   	   ObjectSetText( "[MPClock] Helsinkil", "Helsinki", TimezoneFontSize, FontName, Location_MktClosed );
   	   ObjectSetText( "[MPClock] Helsinkit", Helsinkis, ClockFontSize, FontName, Clock_MktClosed );
   	   }
      }
   if(ShowBerlin)
      {
   	if (TimeDayOfWeek(Berlin) != 0 && TimeDayOfWeek(Berlin) != 6 
   	&& TimeHour(Berlin) >= LocalOpenHour && TimeHour(Berlin) < LocalCloseHour)
      	{
   	   ObjectSetText( "[MPClock] Berlinl", "Frankfurt", TimezoneFontSize, FontName, Location_MktOpen );
   	   ObjectSetText( "[MPClock] Berlint", Berlins, ClockFontSize, FontName, Clock_MktOpen );
      	}
   	else
      	{
   	   ObjectSetText( "[MPClock] Berlinl", "Frankfurt", TimezoneFontSize, FontName, Location_MktClosed );
   	   ObjectSetText( "[MPClock] Berlint", Berlins, ClockFontSize, FontName, Clock_MktClosed );
      	}
      }
   if(ShowLondon)
      {
   	if (TimeDayOfWeek(London) != 0 && TimeDayOfWeek(London) != 6 
   	&& TimeHour(London) >= LocalOpenHour && TimeHour(London) < LocalCloseHour)
   	   {
      	ObjectSetText( "[MPClock] Londonl", "London", TimezoneFontSize, FontName, Location_MktOpen );
      	ObjectSetText( "[MPClock] Londont", Londons, ClockFontSize, FontName, Clock_MktOpen );
   	   }
   	else
   	   {
   	   ObjectSetText( "[MPClock] Londonl", "London", TimezoneFontSize, FontName, Location_MktClosed );
   	   ObjectSetText( "[MPClock] Londont", Londons, ClockFontSize, FontName, Clock_MktClosed );
   	   }
      }
   if(ShowUTC_GMT)
      {
   	ObjectSetText( "[MPClock] utcl", "UTC/GMT", TimezoneFontSize, FontName, Location_MktClosed );
   	ObjectSetText( "[MPClock] utct", UTCs, ClockFontSize, FontName, Clock_MktClosed );
      }
   if(ShowBrazil)
      {
   	if (TimeDayOfWeek(Brazil) != 0 && TimeDayOfWeek(Brazil) != 6 
   	&& TimeHour(Brazil) >= LocalOpenHour && TimeHour(Brazil) < LocalCloseHour)
   	   {
   	   ObjectSetText( "[MPClock] Brazill", "Brazil", TimezoneFontSize, FontName, Location_MktOpen );
   	   ObjectSetText( "[MPClock] Brazilt", Brazils, ClockFontSize, FontName, Clock_MktOpen );
      	}
   	else
      	{
   	   ObjectSetText( "[MPClock] Brazill", "Brazil", TimezoneFontSize, FontName, Location_MktClosed );
   	   ObjectSetText( "[MPClock] Brazilt", Brazils, ClockFontSize, FontName, Clock_MktClosed );
      	}
      }
   if(ShowNewYork)
      {
   	if (TimeDayOfWeek(NewYork) != 0 && TimeDayOfWeek(NewYork) != 6 
   	&& TimeHour(NewYork) >= LocalOpenHour && TimeHour(NewYork) < LocalCloseHour)
   	   {
   	   ObjectSetText( "[MPClock] NewYorkl", "New York", TimezoneFontSize, FontName, Location_MktOpen );
      	ObjectSetText( "[MPClock] NewYorkt", NewYorks, ClockFontSize, FontName, Clock_MktOpen );
   	   }
   	else
   	   {
      	ObjectSetText( "[MPClock] NewYorkl", "New York", TimezoneFontSize, FontName, Location_MktClosed );
      	ObjectSetText( "[MPClock] NewYorkt", NewYorks, ClockFontSize, FontName, Clock_MktClosed );
         }
      }
   if(ShowCentral)
      {
   	if (TimeDayOfWeek(Central) != 0 && TimeDayOfWeek(Central) != 6 
   	&& TimeHour(Central) >= LocalOpenHour && TimeHour(Central) < LocalCloseHour)
   	   {
   	   ObjectSetText( "[MPClock] Centrall", "Central", TimezoneFontSize, FontName, Location_MktOpen );
   	   ObjectSetText( "[MPClock] Centralt", Centrals, ClockFontSize, FontName, Clock_MktOpen );
   	   }
   	else
   	   {
   	   ObjectSetText( "[MPClock] Centrall", "Central", TimezoneFontSize, FontName, Location_MktClosed );
   	   ObjectSetText( "[MPClock] Centralt", Centrals, ClockFontSize, FontName, Clock_MktClosed );
   	   }
      }
   if(ShowMexico)
      {
   	if (TimeDayOfWeek(Mexico) != 0 && TimeDayOfWeek(Mexico) != 6 
   	&& TimeHour(Mexico) >= LocalOpenHour && TimeHour(Mexico) < LocalCloseHour)
   	   {
   	   ObjectSetText( "[MPClock] Mexicol", "Mexico", TimezoneFontSize, FontName, Location_MktOpen );
   	   ObjectSetText( "[MPClock] Mexicot", Mexicos, ClockFontSize, FontName, Clock_MktOpen );
   	   }
   	else
   	   {
   	   ObjectSetText( "[MPClock] Mexicol", "Mexico", TimezoneFontSize, FontName, Location_MktClosed );
   	   ObjectSetText( "[MPClock] Mexicot", Mexicos, ClockFontSize, FontName, Clock_MktClosed );
   	   }
      }
   if(ShowMountain)
      {
   	if (TimeDayOfWeek(Mountain) != 0 && TimeDayOfWeek(Mountain) != 6 
   	&& TimeHour(Mountain) >= LocalOpenHour && TimeHour(Mountain) < LocalCloseHour)
      	{
   	   ObjectSetText( "[MPClock] Mountainl", "Mountain", TimezoneFontSize, FontName, Location_MktOpen );
   	   ObjectSetText( "[MPClock] Mountaint", Mountains, ClockFontSize, FontName, Clock_MktOpen );
   	   }
   	else
   	   {
   	   ObjectSetText( "[MPClock] Mountainl", "Mountain", TimezoneFontSize, FontName, Location_MktClosed );
   	   ObjectSetText( "[MPClock] Mountaint", Mountains, ClockFontSize, FontName, Clock_MktClosed );
   	   }
      }
   if(ShowPacific)
      {
   	if (TimeDayOfWeek(Pacific) != 0 && TimeDayOfWeek(Pacific) != 6 
   	&& TimeHour(Pacific) >= LocalOpenHour && TimeHour(Pacific) < LocalCloseHour)
   	   {
   	   ObjectSetText( "[MPClock] Pacificl", "Pacific", TimezoneFontSize, FontName, Location_MktOpen );
   	   ObjectSetText( "[MPClock] Pacifict", Pacifics, ClockFontSize, FontName, Clock_MktOpen );
   	   }
   	else
      	{
   	   ObjectSetText( "[MPClock] Pacificl", "Pacific", TimezoneFontSize, FontName, Location_MktClosed );
   	   ObjectSetText( "[MPClock] Pacifict", Pacifics, ClockFontSize, FontName, Clock_MktClosed );
      	}
      }
   if ( ShowLocal)
      {
      ObjectSetText( "[MPClock] Locall", "Local", TimezoneFontSize, FontName, Location_MktClosed );
      ObjectSetText( "[MPClock] Localt", Locals, ClockFontSize, FontName, Clock_MktClosed );
      }

   return(0);
   } 

//+------------------------------------------------------------------------------------------+
//| Subroutine to make city and time labels                                                  |
//+------------------------------------------------------------------------------------------+
int ObjectMakeLabel( string n, int xoff, int yoff )
   {
   ObjectCreate( n, OBJ_LABEL, 0, 0, 0 );
   ObjectSet( n, OBJPROP_CORNER, 0 );
   ObjectSet( n, OBJPROP_XDISTANCE, xoff );
   ObjectSet( n, OBJPROP_YDISTANCE, yoff );
   ObjectSet( n, OBJPROP_BACK, false ); 
   } 

//+------------------------------------------------------------------------------------------+
//| Subroutine to creat additional background components                                     |
//+------------------------------------------------------------------------------------------+
int createBackground (string backName, string text, int Bfontsize, int xB, int yB)
{
	if (ObjectFind(backName) == -1){
      ObjectCreate(backName, OBJ_LABEL, 0, 0, 0, 0, 0);}         
   ObjectSetText(backName, text, Bfontsize, "Webdings");      
   ObjectSet(backName, OBJPROP_CORNER, 0);
   ObjectSet(backName, OBJPROP_BACK, false);
   ObjectSet(backName, OBJPROP_XDISTANCE, xB);
   ObjectSet(backName, OBJPROP_YDISTANCE, yB );    
   ObjectSet(backName, OBJPROP_COLOR, Background);
} 

//+------------------------------------------------------------------------------------------+
//| Subroutine to convert time to a string for time labels                                   |
//+------------------------------------------------------------------------------------------+
string TimeToString( datetime when )
   {
   string timeStr;
   int hour = TimeHour( when );
   if ( !Display_Times_With_AMPM ) 
      {
      if (Display_Time_With_Seconds) timeStr = (TimeToStr( when, TIME_MINUTES| TIME_SECONDS));
      else timeStr = (TimeToStr( when, TIME_MINUTES));
      }
   else
      { 
      // User wants 12HourTime format with "AM" or "PM".   
      // FYI, if >12:00, subtract 12 hours in seconds which is 12*60*60=43200
      if (Display_Time_With_Seconds)
         {
         if ( hour >  12 || hour == 0) timeStr = TimeToStr( (when - 43200), TIME_MINUTES| TIME_SECONDS);
         else timeStr = TimeToStr( when, TIME_MINUTES|TIME_SECONDS);
         if ( hour >= 12) timeStr = StringConcatenate(timeStr, " PM");
         else timeStr = StringConcatenate(timeStr, " AM");
         }
      else
         {
         if ( hour >  12 || hour == 0) timeStr = TimeToStr( (when - 43200), TIME_MINUTES);
         else timeStr = TimeToStr( when, TIME_MINUTES);
         if ( hour >= 12) timeStr = StringConcatenate(timeStr, " PM");
         else timeStr = StringConcatenate(timeStr, " AM");
         }
      }
   return (timeStr);
   }

//+------------------------------------------------------------------------------------------+
//| Subroutine to retrieve date and time from time array                                     |
//+------------------------------------------------------------------------------------------+
datetime TimeArrayToTime(int& LocalTimeArray[])
   {  
   int    nYear,nMonth,nDOW,nDay,nHour,nMin,nSec,nMilliSec;
   nYear=LocalTimeArray[0]&0x0000FFFF;
   nMonth=LocalTimeArray[0]>>16;
   //nDOW=LocalTimeArray[1]&0x0000FFFF;
   nDay=LocalTimeArray[1]>>16;
   nHour=LocalTimeArray[2]&0x0000FFFF;
   nMin=LocalTimeArray[2]>>16;
   nSec=LocalTimeArray[3]&0x0000FFFF;
   nMilliSec=LocalTimeArray[3]>>16;
   string LocalTimeS = FormatDateTime(nYear,nMonth,nDay,nHour,nMin,nSec);
   datetime Local_Time = StrToTime( LocalTimeS );
   return(Local_Time);
   } 

//+------------------------------------------------------------------------------------------+
//| Subroutine to format time labels                                                         |
//+------------------------------------------------------------------------------------------+
string FormatDateTime(int nYear,int nMonth,int nDay,int nHour,int nMin,int nSec)
   {
   string sMonth,sDay,sHour,sMin,sSec;
   sMonth=100+nMonth;
   sMonth=StringSubstr(sMonth,1);
   sDay=100+nDay;
   sDay=StringSubstr(sDay,1);
   sHour=100+nHour;
   sHour=StringSubstr(sHour,1);
   sMin=100+nMin;
   sMin=StringSubstr(sMin,1);
   sSec=100+nSec;
   sSec=StringSubstr(sSec,1);
   return(StringConcatenate(nYear,".",sMonth,".",sDay," ",sHour,":",sMin,":",sSec));
   } 

//+------------------------------------------------------------------------------------------+
//| Subroutine to get time zone information for each city                                    |
//+------------------------------------------------------------------------------------------+
void GetAllTimeZoneInfo(int& AucklandTZInfoArray[], int& SydneyTZInfoArray[], 
   int& TokyoTZInfoArray[], int& ChinaTZInfoArray[], int& JakartaTZInfoArray[], 
   int& IndiaTZInfoArray[], int& DubaiTZInfoArray[], int& MoscowTZInfoArray[], 
   int& IsraelTZInfoArray[], int& HelsinkiTZInfoArray[], int& BerlinTZInfoArray[], 
   int& LondonTZInfoArray[], int& BrazilTZInfoArray[], int& NewYorkTZInfoArray[], 
   int& CentralTZInfoArray[], int& MexicoTZInfoArray[], int& MountainTZInfoArray[], 
   int& PacificTZInfoArray[], int& LocalTZInfoArray[])
   
   {
   AucklandTZInfoArray[0] = -720;
   AucklandTZInfoArray[17] = 262144; //4<<16 == 262144  April
   AucklandTZInfoArray[18] = 65536;  //1<<16 == 65536   1st Sunday
   AucklandTZInfoArray[19] = 3;
   AucklandTZInfoArray[20] = 0;
   AucklandTZInfoArray[21] = 0;
   AucklandTZInfoArray[38] = 589824; //9<<16 == 589824  September
   AucklandTZInfoArray[39] = 327680; //5<<16 == 327680  5th/Last Sunday
   AucklandTZInfoArray[40] = 2;
   AucklandTZInfoArray[41] = 0;
   AucklandTZInfoArray[42] = -60;
   
   SydneyTZInfoArray[0] = -600;
   SydneyTZInfoArray[17] = 262144; //4<<16 == 262144  April
   SydneyTZInfoArray[18] = 65536;  //1<<16 == 65536   1st Sunday
   SydneyTZInfoArray[19] = 3;
   SydneyTZInfoArray[20] = 0;
   SydneyTZInfoArray[21] = 0;
   SydneyTZInfoArray[38] = 655360; //10<<16 == 655360  October
   SydneyTZInfoArray[39] = 65536;  //1<<16 == 65536    1st Sunday
   SydneyTZInfoArray[40] = 2;
   SydneyTZInfoArray[41] = 0;
   SydneyTZInfoArray[42] = -60;
   
   // FYI Tokyo = Seoul
   TokyoTZInfoArray[0] = -540;
   TokyoTZInfoArray[17] = 0; 
   TokyoTZInfoArray[18] = 0;
   TokyoTZInfoArray[19] = 0;
   TokyoTZInfoArray[20] = 0;
   TokyoTZInfoArray[21] = 0;
   TokyoTZInfoArray[38] = 0;
   TokyoTZInfoArray[39] = 0;
   TokyoTZInfoArray[40] = 0;
   TokyoTZInfoArray[41] = 0;
   TokyoTZInfoArray[42] = 0;
   
   // FYI, Beijing = Perth, Singapore, Taipei, Hong Kong
   ArrayCopy(ChinaTZInfoArray, TokyoTZInfoArray);
   ChinaTZInfoArray[0] = -480;
   
   // FYI, Jakarta = Bangkok
   ArrayCopy(JakartaTZInfoArray, TokyoTZInfoArray);
   JakartaTZInfoArray[0] = -420;
   
   ArrayCopy(IndiaTZInfoArray, TokyoTZInfoArray);
   IndiaTZInfoArray[0] = -330;  // NOTE! Top of the hour is 30 min off most world timezones.
   
   ArrayCopy(DubaiTZInfoArray, TokyoTZInfoArray);
   DubaiTZInfoArray[0] = -240;
 
   IsraelTZInfoArray[0] = -120;
   IsraelTZInfoArray[17] = 589824;  //9<<16 == 589824  September
   IsraelTZInfoArray[18] = 131072;  //2<<16  = 131072
   IsraelTZInfoArray[19] = 2;
   IsraelTZInfoArray[20] = 0;
   IsraelTZInfoArray[21] = 0;
   IsraelTZInfoArray[38] = 196608; //3<<16 == 196608  March
   IsraelTZInfoArray[39] = 327685; //5<<16  = 327680 AND "5" (*IF* Sunday is "0", is "5" Friday??)
   IsraelTZInfoArray[40] = 2;
   IsraelTZInfoArray[41] = 0;
   IsraelTZInfoArray[42] = -60;
  
   LondonTZInfoArray[0] = 0;
   LondonTZInfoArray[17] = 655360; //10<<16 == 655360  October
   LondonTZInfoArray[18] = 327680; //5<<16 == 327680.  5th/Last Sunday. 
   LondonTZInfoArray[19] = 2;
   LondonTZInfoArray[20] = 0;
   LondonTZInfoArray[21] = 0;
   LondonTZInfoArray[38] = 196608; //3<<16 == 196608  March
   LondonTZInfoArray[39] = 327680; //5<<16 == 327680  5th/Last Sunday
   LondonTZInfoArray[40] = 1;
   LondonTZInfoArray[41] = 0;
   LondonTZInfoArray[42] = -60;
   
   ArrayCopy(MoscowTZInfoArray, LondonTZInfoArray);
   MoscowTZInfoArray[0] = -180;
   MoscowTZInfoArray[19] = 3;
   MoscowTZInfoArray[40] = 2;
   
   // FYI, Helsinki = Athens
   ArrayCopy(HelsinkiTZInfoArray, LondonTZInfoArray);
   HelsinkiTZInfoArray[0] = -120;
   HelsinkiTZInfoArray[19] = 4;
   HelsinkiTZInfoArray[40] = 3;
   
   // FYI, Berlin = Belgrade, Brussels, Paris, Sarajevo
   ArrayCopy(BerlinTZInfoArray, MoscowTZInfoArray);
   BerlinTZInfoArray[0] = -60;
   
   // NOTE! Brazil's ST/DST is likely a hardcoded date which will need updating EVERY YEAR!
   BrazilTZInfoArray[0] = 180;
   BrazilTZInfoArray[17] = 131072;   //2<<16  = 131072
   BrazilTZInfoArray[18] = 196614;   //3<<16 == 196608 AND 6, so, Saturday?? or Mar 6?
   BrazilTZInfoArray[19] = 3866647;
   BrazilTZInfoArray[20] = 65470523;
   BrazilTZInfoArray[21] = 0;
   BrazilTZInfoArray[38] = 655360;   //10<<16 == 655360
   BrazilTZInfoArray[39] = 196614;
   BrazilTZInfoArray[40] = 3866647;
   BrazilTZInfoArray[41] = 65470523;
   BrazilTZInfoArray[42] = -60;
   
   //ArrayCopy(BuenasAriesTZInfoArray, BuenasAriesTZInfoArray);
   //BuenasAriesTZInfoArray[0] = 180;
   
   NewYorkTZInfoArray[0] = 300;
   NewYorkTZInfoArray[17] = 720896; //wYear = 0. wMonth = 11, and 11<<16 == 720896
   NewYorkTZInfoArray[18] = 65536;  //wDOW = 0 = Sunday. nDay = 1 and 1<<16 == 65536 
   NewYorkTZInfoArray[19] = 2;
   NewYorkTZInfoArray[20] = 0;
   NewYorkTZInfoArray[21] = 0;
   NewYorkTZInfoArray[38] = 196608; //3<<16 == 196608  March
   NewYorkTZInfoArray[39] = 131072; //2<<16 == 131072  2nd Sunday
   NewYorkTZInfoArray[40] = 2;
   NewYorkTZInfoArray[41] = 0;
   NewYorkTZInfoArray[42] = -60;
   
   ArrayCopy(CentralTZInfoArray, NewYorkTZInfoArray);
   CentralTZInfoArray[0] = 360;
 
   MexicoTZInfoArray[0] = 360;
   MexicoTZInfoArray[17] = 655360; //10<<16 == 655360  October
   MexicoTZInfoArray[18] = 327680; //5<<16 == 327680  5th/Last Sunday
   MexicoTZInfoArray[19] = 2;
   MexicoTZInfoArray[20] = 0;
   MexicoTZInfoArray[21] = 0;
   MexicoTZInfoArray[38] = 262144; //4<<16 == 262144  April
   MexicoTZInfoArray[39] = 65536;  //1<<16 == 65536   1st Sunday
   MexicoTZInfoArray[40] = 2;
   MexicoTZInfoArray[41] = 0;
   MexicoTZInfoArray[42] = -60;
   
   ArrayCopy(MountainTZInfoArray, NewYorkTZInfoArray);
   MountainTZInfoArray[0] = 420;
   
   ArrayCopy(PacificTZInfoArray, NewYorkTZInfoArray);
   PacificTZInfoArray[0] = 480;   
   } 

//+------------------------------------------------------------------------------------------+
//| Custom Indicator End                                                                     |
//+------------------------------------------------------------------------------------------+

