//+------------------------------------------------------------------+
//|                                                                  |
//|                     SonicR Dragon.mq4                       |
//|                                                                  |
//+------------------------------------------------------------------+ 
#property copyright "Traderathome, Copyright @ 2010"
#property link      "email: traderathome@msn.com"
/*--------------------------------------------------------------------
SonicR Dragon:

This indicator displays a band consisting of three MA lines based on
user "Control" inputs for the moving average period and time frame.
The MA lines can be set to "AutoAdjust" period on charts with time
frames other than the "Control_TF" so that the MA lines will display
at the same approximate price levels regardless of the chart TF. 

The indicator can be turned on/off, preserving all your indicator
settings for the chart.  You can set a TF above which the indicator 
will not display, even though it is turned on.  When AutoAdjust is
on you can set a Max TF and a Min TF, establishing a range of TFs for
AutoAdjusting.  Beyond those TFs, the AutoAdjust will disengauge.

The width of the band can be expanded or contracted using the 
"Dragon_Pip_Width_Adjustment".  for example, entering "1" will expand
the upper and lower lines out from the center by one pip each.  And
entering "-1" will contract the upper and lower lines in towards the 
center line by one pip each.
                                                        - Traderathome
---------------------------------------------------------------------- 
Suggested Settings:             White Chart      Black Chart

#property indicator_color1      DodgerBlue       LightSlateGray 
#property indicator_color2      Red              LightSlateGray
#property indicator_color3      DodgerBlue       LightSlateGray

#property indicator_width1      1                1
#property indicator_width2      1                1
#property indicator_width3      1                1

#property indicator_style1      STYLE_SOLID      STYLE_SOLID
#property indicator_style2      STYLE_SOLID      STYLE_SOLID
#property indicator_style3      STYLE_SOLID      STYLE_SOLID 
                                                                                                      
---------------------------------------------------------------------*/
#property indicator_chart_window

#property indicator_buffers  3
#property indicator_color1   DodgerBlue 
#property indicator_color2   Red
#property indicator_color3   DodgerBlue

#property indicator_width1   1
#property indicator_width2   1
#property indicator_width3   1

#property indicator_style1   STYLE_SOLID
#property indicator_style2   STYLE_SOLID
#property indicator_style3   STYLE_SOLID


//global external inputs
extern string  Part_1                       = "Display On/Off Settings:";
extern bool    Indicator_ON                 = true;
extern string  Indicator_Display_MaxTF      = "D";
extern string  TF_Choices                   = "M1, M5, M15, M30, H1, H4, D, W, M";

extern string  __                           = "";
extern string  Part_2                       = "Dragon Settings:";
extern bool    Dragon_AutoAdjust_On         = true;
extern string  Dragon_AutoAdjust_MaxTF      = "M30";
extern string  Dragon_AutoAdjust_MinTF      = "M30";
extern int     Control_MA                   = 34;
extern string  Control_TF                   = "M15";
extern int     MA_Type                      = 1;
extern string  MA_Type_Choices              = "0=SMA, 1=EMA";
extern int     Centerline                   = 0;
extern string  Centerline_Choices           = "0=CLOSE, 4=MEDIAN";
extern double  Dragon_Pip_Width_Adjustment  = 0;

//global buffers and variables
double MAhighBuffer[], MAcenterBuffer[], MAlowBuffer[],Z;
int    MA_Period,E,F1,F2,G,i,counted_bars,limit,BarShift,Factor;
bool   FLAG_deinit;

//+------------------------------------------------------------------+
//| Initialize                                                       |
//+------------------------------------------------------------------+
int init()
   {
   FLAG_deinit = false;   
   //Select proper Factor for Dragon expansion number
        if (Digits == 2) Factor = 100;
   else if (Digits == 3) Factor = 1000;         
   else if (Digits == 4) Factor = 10000;    
   else if (Digits == 5) Factor = 100000;    
   //Dragon buffers 
   SetIndexBuffer(0, MAhighBuffer);
   SetIndexBuffer(1, MAcenterBuffer);
   SetIndexBuffer(2, MAlowBuffer);
   //initiate drawing styles for buffer values
   SetIndexStyle(0, DRAW_LINE);
   SetIndexStyle(1, DRAW_LINE);   
   SetIndexStyle(2, DRAW_LINE);
   //Set the indicator values
   SetIndexEmptyValue(0,0);
   SetIndexEmptyValue(1,0);
   SetIndexEmptyValue(2,0);        

   MA_Period= Control_MA;              
   if (Dragon_AutoAdjust_On) 
      {         
      //Get F value for maxTF to AutoAdjust
             if(Dragon_AutoAdjust_MaxTF== "M1") {F1= 1;}
      else  {if(Dragon_AutoAdjust_MaxTF== "M5") {F1= 5;}
      else  {if(Dragon_AutoAdjust_MaxTF== "M15"){F1= 15;}
      else  {if(Dragon_AutoAdjust_MaxTF== "M30"){F1= 30;}
      else  {if(Dragon_AutoAdjust_MaxTF== "H1") {F1= 60;}
      else  {if(Dragon_AutoAdjust_MaxTF== "H4") {F1= 240;}
      else  {if(Dragon_AutoAdjust_MaxTF== "D")  {F1= 1440;}
      else  {if(Dragon_AutoAdjust_MaxTF== "W")  {F1= 10080;}   
      else  {if(Dragon_AutoAdjust_MaxTF== "M")  {F1= 43200;} }}}}}}}}
         
      //Get F2 value for minTF to AutoAdjust
             if(Dragon_AutoAdjust_MinTF== "M1") {F2= 1;}
      else  {if(Dragon_AutoAdjust_MinTF== "M5") {F2= 5;}
      else  {if(Dragon_AutoAdjust_MinTF== "M15"){F2= 15;}
      else  {if(Dragon_AutoAdjust_MinTF== "M30"){F2= 30;}
      else  {if(Dragon_AutoAdjust_MinTF== "H1") {F2= 60;}
      else  {if(Dragon_AutoAdjust_MinTF== "H4") {F2= 240;}
      else  {if(Dragon_AutoAdjust_MinTF== "D")  {F2= 1440;}
      else  {if(Dragon_AutoAdjust_MinTF== "W")  {F2= 10080;}
      else  {if(Dragon_AutoAdjust_MinTF== "M")  {F2= 43200;} }}}}}}}} 
         
      //When TF is in F1-F2 range, get G value & AutoAdjust MA period 
      if(Period() <= F1 && Period() >= F2) 
         {   
               if(Control_TF== "M1") {G= 1;}
         else {if(Control_TF== "M5") {G= 5;}
         else {if(Control_TF== "M15"){G= 15;}
         else {if(Control_TF== "M30"){G= 30;}
         else {if(Control_TF== "H1") {G= 60;}
         else {if(Control_TF== "H4") {G= 240;}
         else {if(Control_TF== "D")  {G= 1440;}
         else {if(Control_TF== "W")  {G= 10080;}
         else {if(Control_TF== "M")  {G= 43200;} }}}}}}}} 
         MA_Period= MathRound(Control_MA * G / Period());
         }                                
      }//End AutoAdjust Routine
   
   //Define Indicator Short Name ------------------------------------- 
   IndicatorShortName("SonicR Dragon ("+DoubleToStr(MA_Period,0)+") ");                     
               
   return(0);
   }
   
//+------------------------------------------------------------------+
//| Deinitialize                                                     |
//+------------------------------------------------------------------+
int deinit()
   {      
   return(0);
   }
   
//+------------------------------------------------------------------+
//| Start Program                                                    |
//+------------------------------------------------------------------+
int start()
   {
   //If Indicator is "Off" deinitialize only once, not every tick-----  
   if(!Indicator_ON) 
      {
      if (!FLAG_deinit) {deinit(); FLAG_deinit = true;}
      return(0);
      }
              
   //Exit if period is greater than-----------------------------------      
         if(Indicator_Display_MaxTF== "M1") {E= 1;}
   else {if(Indicator_Display_MaxTF== "M5") {E= 5;}
   else {if(Indicator_Display_MaxTF== "M15"){E= 15;}
   else {if(Indicator_Display_MaxTF== "M30"){E= 30;}
   else {if(Indicator_Display_MaxTF== "H1") {E= 60;}
   else {if(Indicator_Display_MaxTF== "H4") {E= 240;}
   else {if(Indicator_Display_MaxTF== "D")  {E= 1440;}
   else {if(Indicator_Display_MaxTF== "W")  {E= 10080;}
   else {if(Indicator_Display_MaxTF== "M")  {E= 43200;}}}}}}}}}
   if(Period() > E) 
      {
      if (!FLAG_deinit) {deinit(); FLAG_deinit = true;}
      return(-1);  
      }
   
   //If indicator is "ON", deinitialize and run-----------------------   
   deinit(); FLAG_deinit = false;
                       
   //Calculate MAs ---------------------------------------------------   
   //check for possible errors
   counted_bars = IndicatorCounted();
   if(counted_bars < 0)  return(-1);   
   //last counted bar will be recounted
   if(counted_bars > 0) counted_bars--;    
   //calculate the three MA lines 
   limit = Bars - counted_bars; 
   Z = Dragon_Pip_Width_Adjustment/Factor;
   for(i = limit - 1; i >= 0; i--)  
      {
      BarShift = iBarShift(NULL,NULL,Time[i],true);         
      MAhighBuffer[i] = iMA(NULL,NULL,MA_Period,0,MA_Type,PRICE_HIGH,BarShift)+Z;       
      MAcenterBuffer[i] = iMA(NULL,NULL,MA_Period,0,MA_Type,Centerline,BarShift); 
      MAlowBuffer[i] = iMA(NULL,NULL,MA_Period ,0,MA_Type,PRICE_LOW,BarShift)-Z;
      }         
             
   return(0);
   }
   
//+------------------------------------------------------------------+
//| End Program                                                      |
//+------------------------------------------------------------------+